#!/usr/bin/perl
#
# makeNetGraph.pl
#
# Reads in graph information and produces a network map
#
# This code is by Foofus (foofus at foofus dot net)

# Default parameters
$colormap="colormap.txt";
$shapes="shapemap.txt";
$status="status.txt";
$dir=".";
$graphAttributes="UNSET";
$defaultColor="yellow";
$defaultShape="invtriangle";
$routerFont="14";
$netFont="12";
$areaWeight=5;
$suffix=".router";
%netHash=("netName"=>"DONE");

# Parse command-line parameters
$numargs=@ARGV;
        foreach $argument (@ARGV)
        {
		chomp($argument);
                @pair=split(/\:/,$argument);
                if (@pair[0] eq "cm")
                {
                        $colormap=@pair[1];
                } elsif (@pair[0] eq "sm")
                {
                        $shapes=@pair[1];
                } elsif (@pair[0] eq "st")
                {
                        $status=@pair[1];
                } elsif (@pair[0] eq "ga")
                {
                        $graphAttributes=@pair[1];
                } elsif (@pair[0] eq "rf")
                {
                        $routerFont=@pair[1];
		} elsif (@pair[0] eq "sf")
		{
			$suffix=@pair[1];
		} elsif (@pair[0] eq "nf")
		{
			$netFont=@pair[1];
		} elsif (@pair[0] eq "dr")
		{
			$dir=@pair[1];
                } else{
                        print "@pair[0] is an unsupported option\n";
                        &showUsage;
                        die;
                }
        } # (foreach)

# Sanity check


# Say hello
print "/* This .dot output was generated by makeNetGraph.pl /*\n";
print "/*    by Foofus.\n";
print "/*    *** NOTE: If you see $defaultColor nodes in the shape of $defaultShape, */\n";
print "/*    ***       then something has gone wrong.  Check your status file... */\n";

# Read in color map, assign ordered values
open(INFILE,"<$colormap") || die "ERROR: Could not open $colormap for reading!\n";
@colors=<INFILE>;
close(INFILE);
print "/* Read in color mappings from $colormap */\n";
%colorHash=("nodeType" => "nodeColor"); 
foreach $color (@colors)
{
	chomp($color);
	@elements=split(/:/,$color);
	$colorHash{@elements[0]}=@elements[1];
} # foreach

# Read in shape list
open(INFILE,"<$shapes") || die "ERROR: Could not open $shapes for reading!\n";
@shapes=<INFILE>;
close(INFILE);
print "/* Read in node shapes from $shapes */\n";
%shapeHash=("nodeType" => "nodeShape"); 
foreach $shape (@shapes)
{
	chomp($shape);
	@elements=split(/:/,$shape);
	$shapeHash{@elements[0]}=@elements[1];
} # foreach

# Get a list of routers (i.e., file names)
print "/* Reading router information files (suffix of \"$suffix\") from \"$dir\" */\n";
opendir(DIR,$dir) || die ("ERROR: Unable to open directory $dir\n");
while ($file=readdir(DIR))
{
        chomp($file);
        if (($file ne ".") and ($file ne "..") and (index($file,$suffix)>0))
        {
                $order++;
                push(@raw,$file);
        } # (is this a pwdump file?)
        @routers=sort(@raw);
} # (whille)


# Make graph header
print "graph G {\n";
if ($graphAttributes ne "UNSET")
{
	print "  $graphAttributes\n";
}

# For each router, draw 1 router node, 1 node for each directly 
# connected network named in the file, and an edge between them
print "  /* Describing routers and their directly connected networks */ \n";
$routerColor=$colorHash{"ROUTER"};
$netColor=$colorHash{"NETWORK"};
$netShape=$shapeHash{"NETWORK"};
$routerShape=$shapeHash{"ROUTER"};
foreach $routerFile (@routers)
{
	# Set up for this pass (if you see these attributes, something is wrong!)
	chomp($routerFile);
	$routerName="";

	# Gather information
	@elements=split(/$suffix/,$routerFile);
	$routerName=@elements[0];
	$router="r$routerName";
	$router=~ tr/./d/;
	
	# Draw router node
	print "  /* $routerName and associated networks */\n";
	print "  $router [label=\"$routerName\",fontname=\"Nimbus\",shape=$routerShape,color=\"$routerColor\",style=filled,fontsize=$routerFont];\n";

	# Read router file contents
	open(INFILE,"<$dir/$routerFile") || die "Could not open $dir/$routerFile for reading!\n";
	@rawNets=<INFILE>;
	close(INFILE);
	@nets=sort(@rawNets);

	# Process networks; draw each only once
	foreach $net (@nets)
	{
		chomp($net);
		$netName=$net;
		$net=~ tr/./d/;
		$net=~ tr/\//s/;
		$net="n".$net;
		#$netName=~ s/\//\\\//;
		if (exists ($netHash{$net}))
		{
			print "  /* $routerName is connected to $net, already depicted */\n";
		} else {
			print "  $net [label=\"$netName\",fontname=\"Courier\",shape=$netShape,color=\"$netColor\",style=filled,fontsize=$netFont];\n";
			$netHash{$net}="DONE";
		}
		print "  $router -- $net;\n";
	} # (foreach)
} # (foreach)

# Make graph footer
print "}\n";

sub showUsage {
	print "makeNetGraph.pl -- v1.0 -- Foofus.\n";
	print "  Usage: makeNetGraph.pl [options]\n";
	print "Available parameters:\n";
	print "  cm: colormap file (NODE_TYPE:color) - default is colormap.txt\n";
	print "  sm: shape map file (NODE_TYPE:shape) - default is shapemap.txt\n";
	print "  dr: directory where router-net files are kept - default is \"$dir\" \n";
	print "  st: status file (ObjectiveNode:STATUS_CODE) - default is status.txt\n";
	print "  ga: any graph attributes you might want to specify - default is none\n";
	print "  rf: font size for a router node (default is $routerFont)\n";
	print "  nf: font size for a network node (default is $netFont)\n";
	print "Output is .dot language, suitable for passing to dot or neato\n";
} # (showUsage)
